From f21a04c024e57d2396c6a8ab78b055de098217ee Mon Sep 17 00:00:00 2001
From: alecpl <alec@alec.pl>
Date: Thu, 10 Nov 2011 02:57:56 -0500
Subject: [PATCH] - Add option to define matching method for addressbook search (#1486564, #1487907)

---
 CHANGELOG                             |    1 
 program/include/rcube_addressbook.php |    6 +
 program/steps/addressbook/import.inc  |    4 
 program/include/rcube_ldap.php        |   52 +++++++++++++---
 program/steps/mail/autocomplete.inc   |    5 +
 program/steps/addressbook/mailto.inc  |    2 
 program/steps/addressbook/save.inc    |    2 
 config/main.inc.php.dist              |    7 ++
 program/include/rcube_contacts.php    |   81 +++++++++++++++++++++------
 program/steps/addressbook/copy.inc    |    4 
 program/steps/addressbook/search.inc  |    5 +
 program/steps/mail/addcontact.inc     |    6 +-
 12 files changed, 133 insertions(+), 42 deletions(-)

diff --git a/CHANGELOG b/CHANGELOG
index cc33b26..b8688b2 100644
--- a/CHANGELOG
+++ b/CHANGELOG
@@ -1,6 +1,7 @@
 CHANGELOG Roundcube Webmail
 ===========================
 
+- Add option to define matching method for addressbook search (#1486564, #1487907)
 - Make email recipients separator configurable
 - Fix so folders with \Noinferiors attribute aren't listed in parent selector
 - Fix handling of curly brackets in URLs (#1488168)
diff --git a/config/main.inc.php.dist b/config/main.inc.php.dist
index 9493b30..ef76c2a 100644
--- a/config/main.inc.php.dist
+++ b/config/main.inc.php.dist
@@ -627,6 +627,13 @@
 // available placeholders: {street}, {locality}, {zipcode}, {country}, {region}
 $rcmail_config['address_template'] = '{street}<br/>{locality} {zipcode}<br/>{country} {region}';
 
+// Matching mode for addressbook search (including autocompletion)
+// 0 - partial (*abc*), default
+// 1 - strict (abc)
+// 2 - prefix (abc*)
+// Note: For LDAP sources fuzzy_search must be enabled to use 'partial' or 'prefix' mode
+$rcmail_config['addressbook_search_mode'] = 0;
+
 // ----------------------------------
 // USER PREFERENCES
 // ----------------------------------
diff --git a/program/include/rcube_addressbook.php b/program/include/rcube_addressbook.php
index 7270f42..2b97dea 100644
--- a/program/include/rcube_addressbook.php
+++ b/program/include/rcube_addressbook.php
@@ -96,12 +96,16 @@
      *
      * @param array   List of fields to search in
      * @param string  Search value
+     * @param int     Matching mode:
+     *                0 - partial (*abc*),
+     *                1 - strict (=),
+     *                2 - prefix (abc*)
      * @param boolean True if results are requested, False if count only
      * @param boolean True to skip the count query (select only)
      * @param array   List of fields that cannot be empty
      * @return object rcube_result_set List of contact records and 'count' value
      */
-    abstract function search($fields, $value, $strict=false, $select=true, $nocount=false, $required=array());
+    abstract function search($fields, $value, $mode=0, $select=true, $nocount=false, $required=array());
 
     /**
      * Count number of available contacts in database
diff --git a/program/steps/addressbook/copy.inc b/program/steps/addressbook/copy.inc
index e07d62a..5e526e1 100644
--- a/program/steps/addressbook/copy.inc
+++ b/program/steps/addressbook/copy.inc
@@ -60,9 +60,9 @@
         // Check if contact exists, if so, we'll need it's ID
         // Note: Some addressbooks allows empty email address field
         if (!empty($a_record['email']))
-            $result = $TARGET->search('email', $a_record['email'], true, true, true);
+            $result = $TARGET->search('email', $a_record['email'], 1, true, true);
         else if (!empty($a_record['name']))
-            $result = $TARGET->search('name', $a_record['name'], true, true, true);
+            $result = $TARGET->search('name', $a_record['name'], 1, true, true);
         else
             $result = new rcube_result_set();
 
diff --git a/program/steps/addressbook/import.inc b/program/steps/addressbook/import.inc
index 1b9aea1..63a6dae 100644
--- a/program/steps/addressbook/import.inc
+++ b/program/steps/addressbook/import.inc
@@ -174,9 +174,9 @@
 
       if (!$replace && $email) {
         // compare e-mail address
-        $existing = $CONTACTS->search('email', $email, false, false);
+        $existing = $CONTACTS->search('email', $email, 1, false);
         if (!$existing->count && $vcard->displayname) {  // compare display name
-          $existing = $CONTACTS->search('name', $vcard->displayname, false, false);
+          $existing = $CONTACTS->search('name', $vcard->displayname, 1, false);
         }
         if ($existing->count) {
           $IMPORT_STATS->skipped++;
diff --git a/program/steps/addressbook/mailto.inc b/program/steps/addressbook/mailto.inc
index 99c022d..c40ecdf 100644
--- a/program/steps/addressbook/mailto.inc
+++ b/program/steps/addressbook/mailto.inc
@@ -31,7 +31,7 @@
     {
         $CONTACTS->set_page(1);
         $CONTACTS->set_pagesize(count($cid) + 2); // +2 to skip counting query
-        $recipients = $CONTACTS->search($CONTACTS->primary_key, $cid, false, true, true, 'email');
+        $recipients = $CONTACTS->search($CONTACTS->primary_key, $cid, 0, true, true, 'email');
     }
 }
 
diff --git a/program/steps/addressbook/save.inc b/program/steps/addressbook/save.inc
index ddbd630..0a2d6db 100644
--- a/program/steps/addressbook/save.inc
+++ b/program/steps/addressbook/save.inc
@@ -162,7 +162,7 @@
   // show notice if existing contacts with same e-mail are found
   $existing = false;
   foreach ($CONTACTS->get_col_values('email', $a_record, true) as $email) {
-      if ($email && ($res = $CONTACTS->search('email', $email, false, false, true)) && $res->count) {
+      if ($email && ($res = $CONTACTS->search('email', $email, 1, false, true)) && $res->count) {
           $OUTPUT->show_message('contactexists', 'notice', null, false);
           break;
       }
diff --git a/program/steps/addressbook/search.inc b/program/steps/addressbook/search.inc
index ea98247..643cc60 100644
--- a/program/steps/addressbook/search.inc
+++ b/program/steps/addressbook/search.inc
@@ -137,6 +137,9 @@
         }
     }
 
+    // Values matching mode
+    $mode = (int) $RCMAIL->config->get('addressbook_search_mode');
+
     // get sources list
     $sources    = $RCMAIL->get_address_sources();
     $search_set = array();
@@ -168,7 +171,7 @@
         $source->set_pagesize(9999);
 
         // get contacts count
-        $result = $source->search($fields, $search, false, false);
+        $result = $source->search($fields, $search, $mode, false);
 
         if (!$result->count) {
             continue;
diff --git a/program/steps/mail/addcontact.inc b/program/steps/mail/addcontact.inc
index dafb276..a4feb7b 100644
--- a/program/steps/mail/addcontact.inc
+++ b/program/steps/mail/addcontact.inc
@@ -50,7 +50,7 @@
       $OUTPUT->show_message('errorsavingcontact', 'error');
       $OUTPUT->send();
     }
-    
+
     $email = rcube_idn_to_ascii($contact['email']);
     if (!check_email($email, false)) {
       $OUTPUT->show_message('emailformaterror', 'error', array('email' => $contact['email']));
@@ -65,13 +65,13 @@
       $error = $CONTACTS->get_error();
       // TODO: show dialog to complete record
       // if ($error['type'] == rcube_addressbook::ERROR_VALIDATE) { }
-      
+
       $OUTPUT->show_message($error['message'] ? $error['message'] : 'errorsavingcontact', 'error');
       $OUTPUT->send();
     }
 
     // check for existing contacts
-    $existing = $CONTACTS->search('email', $contact['email'], true, false);
+    $existing = $CONTACTS->search('email', $contact['email'], 1, false);
 
     if ($done = $existing->count)
       $OUTPUT->show_message('contactexists', 'warning');
diff --git a/program/steps/mail/autocomplete.inc b/program/steps/mail/autocomplete.inc
index 8b13f57..c2df0b7 100644
--- a/program/steps/mail/autocomplete.inc
+++ b/program/steps/mail/autocomplete.inc
@@ -41,6 +41,7 @@
 
 
 $MAXNUM = (int)$RCMAIL->config->get('autocomplete_max', 15);
+$mode   = (int) $RCMAIL->config->get('addressbook_search_mode');
 $search = get_input_value('_search', RCUBE_INPUT_GPC, true);
 $source = get_input_value('_source', RCUBE_INPUT_GPC);
 $sid    = get_input_value('_id', RCUBE_INPUT_GPC);
@@ -58,7 +59,7 @@
     $abook = $RCMAIL->get_address_book($id);
     $abook->set_pagesize($MAXNUM);
 
-    if ($result = $abook->search(array('email','name'), $search, false, true, true, 'email')) {
+    if ($result = $abook->search(array('email','name'), $search, $mode, true, true, 'email')) {
       while ($sql_arr = $result->iterate()) {
         // Contact can have more than one e-mail address
         $email_arr = (array)$abook->get_col_values('email', $sql_arr, true);
@@ -82,7 +83,7 @@
     }
 
     // also list matching contact groups
-    if ($abook->groups) {
+    if ($abook->groups && count($contacts) < $MAXNUM) {
       foreach ($abook->list_groups($search) as $group) {
         $abook->reset();
         $abook->set_group($group['ID']);

diff --git a/program/include/rcube_contacts.php b/program/include/rcube_contacts.php
index e822d2c..fe600e0 100644
--- a/program/include/rcube_contacts.php
+++ b/program/include/rcube_contacts.php
@@ -177,12 +177,12 @@
             " AND contactgroup_id=?".
             " AND user_id=?",
             $group_id, $this->user_id);
-            
+
         if ($sql_result && ($sql_arr = $this->db->fetch_assoc($sql_result))) {
             $sql_arr['ID'] = $sql_arr['contactgroup_id'];
             return $sql_arr;
         }
-        
+
         return null;
     }
 
@@ -268,14 +268,17 @@
      *
      * @param mixed   $fields   The field name of array of field names to search in
      * @param mixed   $value    Search value (or array of values when $fields is array)
      * @param boolean $strict   True for strict (=), False for partial (LIKE) matching
      * @param boolean $select   True if results are requested, False if count only
      * @param boolean $nocount  True to skip the count query (select only)
      * @param array   $required List of fields that cannot be empty
      *
      * @return object rcube_result_set Contact records and 'count' value
      */
-    function search($fields, $value, $strict=false, $select=true, $nocount=false, $required=array())
+    function search($fields, $value, $mode=0, $select=true, $nocount=false, $required=array())
     {
         if (!is_array($fields))
             $fields = array($fields);
@@ -283,6 +286,7 @@
             $required = array($required);
 
         $where = $and_where = array();
+        $mode = intval($mode);
 
         foreach ($fields as $idx => $col) {
             // direct ID search
@@ -295,26 +299,56 @@
             // fulltext search in all fields
             else if ($col == '*') {
                 $words = array();
-                foreach (explode(" ", self::normalize_string($value)) as $word)
-                    $words[] = $this->db->ilike('words', '%'.$word.'%');
+                foreach (explode(" ", self::normalize_string($value)) as $word) {
+                    switch ($mode) {
+                    case 1: // strict
+                        $words[] = '(' . $this->db->ilike('words', $word.' %')
+                            . ' OR ' . $this->db->ilike('words', '% '.$word.' %')
+                            . ' OR ' . $this->db->ilike('words', '% '.$word) . ')';
+                        break;
+                    case 2: // prefix
+                        $words[] = '(' . $this->db->ilike('words', $word.'%')
+                            . ' OR ' . $this->db->ilike('words', '% '.$word.'%') . ')';
+                        break;
+                    default: // partial
+                        $words[] = $this->db->ilike('words', '%'.$word.'%');
+                    }
+                }
                 $where[] = '(' . join(' AND ', $words) . ')';
             }
             else {
                 $val = is_array($value) ? $value[$idx] : $value;
                 // table column
                 if (in_array($col, $this->table_cols)) {
-                    if ($strict) {
+                    switch ($mode) {
+                    case 1: // strict
                         $where[] = $this->db->quoteIdentifier($col).' = '.$this->db->quote($val);
-                    }
-                    else {
+                        break;
+                    case 2: // prefix
+                        $where[] = $this->db->ilike($col, $val.'%');
+                        break;
+                    default: // partial
                         $where[] = $this->db->ilike($col, '%'.$val.'%');
                     }
                 }
                 // vCard field
                 else {
                     if (in_array($col, $this->fulltext_cols)) {
-                        foreach (explode(" ", self::normalize_string($val)) as $word)
-                            $words[] = $this->db->ilike('words', '%'.$word.'%');
+                        foreach (explode(" ", self::normalize_string($val)) as $word) {
+                            switch ($mode) {
+                            case 1: // strict
+                                $words[] = '(' . $this->db->ilike('words', $word.' %')
+                                    . ' OR ' . $this->db->ilike('words', '% '.$word.' %')
+                                    . ' OR ' . $this->db->ilike('words', '% '.$word) . ')';
+                                break;
+                            case 2: // prefix
+                                $words[] = '(' . $this->db->ilike('words', $word.'%')
+                                    . ' OR ' . $this->db->ilike('words', ' '.$word.'%') . ')';
+                                break;
+                            default: // partial
+                                $words[] = $this->db->ilike('words', '%'.$word.'%');
+                            }
+                        }
                         $where[] = '(' . join(' AND ', $words) . ')';
                     }
                     if (is_array($value))
@@ -362,13 +396,24 @@
                         $search  = $post_search[$colname];
                         foreach ((array)$row[$col] as $value) {
                             // composite field, e.g. address
-                            if (is_array($value)) {
-                                $value = implode($value);
-                            }
-                            $value = mb_strtolower($value);
-                            if (($strict && $value == $search) || (!$strict && strpos($value, $search) !== false)) {
-                                $found[$colname] = true;
-                                break;
+                            foreach ((array)$value as $val) {
+                                $val = mb_strtolower($val);
+                                switch ($mode) {
+                                case 1:
+                                    $got = ($val == $search);
+                                    break;
+                                case 2:
+                                    $got = ($search == substr($val, 0, strlen($search)));
+                                    break;
+                                default:
+                                    $got = (strpos($val, $search) !== false);
+                                    break;
+                                }
+
+                                if ($got) {
+                                    $found[$colname] = true;
+                                    break 2;
+                                }
                             }
                         }
                     }
@@ -268,14 +268,17 @@
      * @param mixed   $fields   The field name of array of field names to search in
      * @param mixed   $value    Search value (or array of values when $fields is array)
-     * @param boolean $strict   True for strict (=), False for partial (LIKE) matching
+     * @param int     $mode     Matching mode:
+     *                          0 - partial (*abc*),
+     *                          1 - strict (=),
+     *                          2 - prefix (abc*)
      * @param boolean $select   True if results are requested, False if count only
      * @param boolean $nocount  True to skip the count query (select only)
diff --git a/program/include/rcube_ldap.php b/program/include/rcube_ldap.php
index 4fb3792..e308ffd 100644
--- a/program/include/rcube_ldap.php
+++ b/program/include/rcube_ldap.php
@@ -698,15 +698,20 @@
      *
      * @param mixed   $fields   The field name of array of field names to search in
      * @param mixed   $value    Search value (or array of values when $fields is array)
      * @param boolean $strict   True for strict, False for partial (fuzzy) matching
      * @param boolean $select   True if results are requested, False if count only
      * @param boolean $nocount  (Not used)
      * @param array   $required List of fields that cannot be empty
      *
      * @return array  Indexed list of contact records and 'count' value
      */
-    function search($fields, $value, $strict=false, $select=true, $nocount=false, $required=array())
+    function search($fields, $value, $mode=0, $select=true, $nocount=false, $required=array())
     {
+        $mode = intval($mode);
+
         // special treatment for ID-based search
         if ($fields == 'ID' || $fields == $this->primary_key)
         {
@@ -738,13 +743,31 @@
                 array_values($this->fieldmap), 0, (int)$this->prop['sizelimit'], (int)$this->prop['timelimit']);
 
             // get all entries of this page and post-filter those that really match the query
+            $search = mb_strtolower($value);
             $this->result = new rcube_result_set(0);
             $entries = ldap_get_entries($this->conn, $this->ldap_result);
+
             for ($i = 0; $i < $entries['count']; $i++) {
                 $rec = $this->_ldap2result($entries[$i]);
-                if (stripos($rec['name'] . $rec['email'], $value) !== false) {
-                    $this->result->add($rec);
-                    $this->result->count++;
+                foreach (array('email', 'name') as $f) {
+                    $val = mb_strtolower($rec[$f]);
+                    switch ($mode) {
+                    case 1:
+                        $got = ($val == $search);
+                        break;
+                    case 2:
+                        $got = ($search == substr($val, 0, strlen($search)));
+                        break;
+                    default:
+                        $got = (strpos($val, $search) !== false);
+                        break;
+                    }
+
+                    if ($got) {
+                        $this->result->add($rec);
+                        $this->result->count++;
+                        break;
+                    }
                 }
             }
 
@@ -753,7 +776,14 @@
 
         // use AND operator for advanced searches
         $filter = is_array($value) ? '(&' : '(|';
-        $wc     = !$strict && $this->prop['fuzzy_search'] ? '*' : '';
+        // set wildcards
+        $wp = $ws = '';
+        if (!empty($this->prop['fuzzy_search']) && $mode != 1) {
+            $ws = '*';
+            if (!$mode) {
+                $wp = '*';
+            }
+        }
 
         if ($fields == '*')
         {
@@ -767,7 +797,7 @@
             if (is_array($this->prop['search_fields']))
             {
                 foreach ($this->prop['search_fields'] as $field) {
-                    $filter .= "($field=$wc" . $this->_quote_string($value) . "$wc)";
+                    $filter .= "($field=$wp" . $this->_quote_string($value) . "$ws)";
                 }
             }
         }
@@ -776,7 +806,7 @@
             foreach ((array)$fields as $idx => $field) {
                 $val = is_array($value) ? $value[$idx] : $value;
                 if ($f = $this->_map_field($field)) {
-                    $filter .= "($f=$wc" . $this->_quote_string($val) . "$wc)";
+                    $filter .= "($f=$wp" . $this->_quote_string($val) . "$ws)";
                 }
             }
         }
@@ -1433,9 +1463,9 @@
 
         $groups = array();
         if ($search) {
-            $search = strtolower($search);
+            $search = mb_strtolower($search);
             foreach ($group_cache as $group) {
-                if (strstr(strtolower($group['name']), $search))
+                if (strpos(mb_strtolower($group['name']), $search) !== false)
                     $groups[] = $group;
             }
         }
@@ -1511,7 +1541,7 @@
                     $groups[$group_id]['email'][] = $ldap_data[$i][$email_attr][$j];
             }
 
-            $group_sortnames[] = strtolower($ldap_data[$i][$sort_attr][0]);
+            $group_sortnames[] = mb_strtolower($ldap_data[$i][$sort_attr][0]);
         }
 
         // recursive call can exit here
@@ -698,15 +698,20 @@
      * @param mixed   $fields   The field name of array of field names to search in
      * @param mixed   $value    Search value (or array of values when $fields is array)
-     * @param boolean $strict   True for strict, False for partial (fuzzy) matching
+     * @param int     $mode     Matching mode:
+     *                          0 - partial (*abc*),
+     *                          1 - strict (=),
+     *                          2 - prefix (abc*)
      * @param boolean $select   True if results are requested, False if count only
      * @param boolean $nocount  (Not used)
--
Gitblit v1.9.1